@@ -42,6 +42,8 @@ struct editor {
   unsigned int section, playseq, position, block, line, time, cmdpage;
   /* Editor modes */
   unsigned int octave, instrument, edit, chord, space;
+  /* Tracks currently skipped in chord mode */
+  unsigned int chordstatus;
   /* Buffers for copy & paste */
   struct block *copyblock, *copytrack, *copyarea;
   /* GUI for this editor */
@@ -71,4 +73,6 @@ void editor_set_block_immediate(struct editor *, int);
 void editor_set_line_immediate(struct editor *, int);
 void editor_set_commandpage_immediate(struct editor *, int);
 
+void editor_midi_input(struct editor *, unsigned char *);
+
 #endif
