@@ -36,29 +36,31 @@ struct sysex {
 };
 
 /* Opens a MIDI device */
-FILE *midi_open(char *);
+int midi_open(char *);
 /* Closes a MIDI device */
-void midi_close(FILE *);
+void midi_close(int);
 /* Stops a note playing on a MIDI channel using requested velocity */
-void midi_note_off(FILE *, unsigned char, unsigned char, unsigned char);
+void midi_note_off(int, unsigned char, unsigned char, unsigned char);
 /* Plays a note on a MIDI channel using requested velocity */
-void midi_note_on(FILE *, unsigned char, unsigned char, unsigned char);
+void midi_note_on(int, unsigned char, unsigned char, unsigned char);
 /* Sets the aftertouch pressure of a note playing on a MIDI channel */
-void midi_aftertouch(FILE *, unsigned char, unsigned char, unsigned char);
+void midi_aftertouch(int, unsigned char, unsigned char, unsigned char);
 /* Sets the MIDI controller value of a MIDI channel */
-void midi_controller(FILE *, unsigned char, unsigned char, unsigned char);
+void midi_controller(int, unsigned char, unsigned char, unsigned char);
 /* Send a program change on a MIDI channel */
-void midi_program_change(FILE *, unsigned char, unsigned char);
+void midi_program_change(int, unsigned char, unsigned char);
 /* Sets the channel pressure of a MIDI channel */
-void midi_channel_pressure(FILE *, unsigned char, unsigned char);
+void midi_channel_pressure(int, unsigned char, unsigned char);
 /* Sets the pitch wheel value of a MIDI channel */
-void midi_pitch_wheel(FILE *, unsigned char, unsigned short);
+void midi_pitch_wheel(int, unsigned char, unsigned short);
 /* Sends a MIDI SysEx message */
-void midi_system_exclusive(FILE *, unsigned char *, unsigned short);
+void midi_system_exclusive(int, unsigned char *, unsigned short);
 /* Send a clock message */
-void midi_clock(FILE *);
+void midi_clock(int);
+/* Receives a MIDI message */
+unsigned char *midi_read(int);
 /* Receives a MIDI SysEx message */
-void midi_read_system_exclusive(FILE *, struct sysex *, int);
+void midi_read_system_exclusive(int, struct sysex *, int);
 /* Sets the length of a MIDI SysEx message */
 void sysex_set_length(struct sysex *, unsigned int);
 /* Sets the auto send flag of a MIDI SysEx message */
