@@ -37,7 +37,9 @@ enum {
   TRACK_STATUS_MIDI_CHANNEL,
   TRACK_STATUS_NOTE,
   TRACK_STATUS_VOLUME,
-  TRACK_STATUS_HOLD
+  TRACK_STATUS_HOLD,
+  TRACK_STATUS_PREVIOUS_COMMAND,
+  TRACK_STATUS_NUMVALUES
 };
 
 /* 128 MIDI controllers plus aftertouch, channel pressure and pitch wheel */
@@ -65,8 +67,8 @@ struct player {
   int threadstatus;
   /* Player thread pointer */
   pthread_t thread;
-  /* MIDI device stream pointer */
-  FILE *midi;
+  /* MIDI device file descriptor */
+  int midi;
   struct editor *editor;
 };
 
