@@ -29,7 +29,7 @@
 #include "midi.h"
 
 enum {
-  COMMAND_MIDI_COMMAND_VALUE=0x00,
+  COMMAND_PREVIOUS_COMMAND_VALUE=0x00,
   COMMAND_PITCH_WHEEL=0x01,
   COMMAND_END_BLOCK=0x02,
   COMMAND_PLAYSEQ_POSITION=0x03,
@@ -114,7 +114,7 @@ void song_check_instrument(struct song *, int);
 /* Converts an MMD2 module to a song */
 struct song *song_convert_MMD2(struct MMD2 *);
 /* Converts MMD command values at given location */
-void song_convert_commands(unsigned char *);
+void song_convert_commands(unsigned char *, struct MMDMIDICmd3x *);
 /* Loads a song from an XML file */
 struct song *song_load(char *);
 /* Parses a song element in an XML file */
