@@ -408,7 +408,7 @@ struct block *block_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 void block_save(struct block *block, int number, xmlNodePtr parent) {
   char c[10];
   int i, j, k;
-  xmlNodePtr node, subnode, lb;
+  xmlNodePtr node, subnode;
 
   if(block==NULL) {
     fprintf(stderr, "block_save() called with null block\n");
@@ -425,8 +425,7 @@ void block_save(struct block *block, int number, xmlNodePtr parent) {
   xmlSetProp(node, "length", c);
   snprintf(c, 10, "%d", block->commandpages);
   xmlSetProp(node, "commandpages", c);
-  lb=xmlNewText("\n");
-  xmlAddChild(node, lb);
+  xmlAddChild(node, xmlNewText("\n"));
   
   /* Notation data */
   for(i=0; i<block->tracks; i++)
@@ -441,8 +440,7 @@ void block_save(struct block *block, int number, xmlNodePtr parent) {
 	xmlSetProp(subnode, "track", c);
 	snprintf(c, 10, "%d", block->notes[2*(block->tracks*j+i)+1]);
 	xmlSetProp(subnode, "instrument", c);
-	lb=xmlNewText("\n");
-	xmlAddChild(node, lb);
+	xmlAddChild(node, xmlNewText("\n"));
       }
   
   /* Command data */
@@ -461,12 +459,10 @@ void block_save(struct block *block, int number, xmlNodePtr parent) {
 	  xmlSetProp(subnode, "commandpage", c);
 	  snprintf(c, 10, "%d", block->commands[k*2*block->tracks*block->length+2*(block->tracks*j+i)+1]);
 	  xmlSetProp(subnode, "value", c);
-	  lb=xmlNewText("\n");
-	  xmlAddChild(node, lb);
+	  xmlAddChild(node, xmlNewText("\n"));
 	}
   
-  lb=xmlNewText("\n");
-  xmlAddChild(parent, lb);
+  xmlAddChild(parent, xmlNewText("\n"));
 }
 
 /* Transposes a block or a part of it */
