@@ -40,8 +40,10 @@ struct editor *editor_open(char *guifilename) {
 
 /* Opens an existing song or creates new */
 void editor_song_open(struct editor *editor, char *filename) {
-  if(editor==NULL)
+  if(editor==NULL) {
+    fprintf(stderr, "editor_song_open() called with null editor\n");
     return;
+  }
 
   /* Close the old player */
   if(editor->player!=NULL)
@@ -72,8 +74,10 @@ void editor_song_open(struct editor *editor, char *filename) {
 
 /* Deinitializes the editor */
 void editor_close(struct editor *editor) {
-  if(editor==NULL)
+  if(editor==NULL) {
+    fprintf(stderr, "editor_close() called with null editor\n");
     return;
+  }
 
   /* Close the GUI */
   if(editor->gui!=NULL)
@@ -107,6 +111,11 @@ void editor_close(struct editor *editor) {
 void editor_set_values(struct editor *editor,
 		       int section, int playseq, int position, int block,
 		       int line, int time, int tempochanged) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_values() called with null editor\n");
+    return;
+  }
+
   if(section!=editor->section &&
      section>=0 && section<editor->song->numsections) {
     gui_set_refresh(editor->gui, GUI_REFRESH_INFO|GUI_REFRESH_SECTIONLIST);
@@ -144,6 +153,11 @@ void editor_set_values(struct editor *editor,
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
 void editor_set_section_immediate(struct editor *editor, int section) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_section_immediate() called with null editor\n");
+    return;
+  }
+
   if(section!=editor->section) {
     if(section<0)
       section=0;
@@ -160,6 +174,11 @@ void editor_set_section_immediate(struct editor *editor, int section) {
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
 void editor_set_playseq_immediate(struct editor *editor, int playseq) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_playseq_immediate() called with null editor\n");
+    return;
+  }
+
   if(playseq!=editor->playseq) {
     if(playseq<0)
       playseq=0;
@@ -176,6 +195,11 @@ void editor_set_playseq_immediate(struct editor *editor, int playseq) {
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
 void editor_set_position_immediate(struct editor *editor, int position) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_position_immediate() called with null editor\n");
+    return;
+  }
+
   if(position!=editor->position) {
     if(position<0)
       position=0;
@@ -192,6 +216,11 @@ void editor_set_position_immediate(struct editor *editor, int position) {
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
 void editor_set_block_immediate(struct editor *editor, int block) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_block_immediate() called with null editor\n");
+    return;
+  }
+
   if(block!=editor->block) {
     if(block<0)
       block=0;
@@ -214,6 +243,11 @@ void editor_set_block_immediate(struct editor *editor, int block) {
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
 void editor_set_line_immediate(struct editor *editor, int line) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_line_immediate() called with null editor\n");
+    return;
+  }
+
   if(line!=editor->line) {
     while(line<0)
       line+=editor->song->blocks[editor->block]->length;
@@ -229,6 +263,11 @@ void editor_set_line_immediate(struct editor *editor, int line) {
 
 /* Changes the position of the editor and refreshes the GUI accordingly */
 void editor_set_commandpage_immediate(struct editor *editor, int cmdpage) {
+  if(editor==NULL) {
+    fprintf(stderr, "editor_set_commandpage_immediate() called with null editor\n");
+    return;
+  }
+
   if(cmdpage!=editor->cmdpage) {
     while(cmdpage<0)
       cmdpage+=editor->song->blocks[editor->block]->commandpages;
@@ -245,7 +284,14 @@ void editor_set_commandpage_immediate(struct editor *editor, int cmdpage) {
 
 /* Refreshes playseq from section and block from position */
 void editor_refresh_playseq_and_block(struct editor *editor) {
-  struct song *song=editor->song;
+  struct song *song;
+
+  if(editor==NULL) {
+    fprintf(stderr, "editor_refresh_playseq_and_block() called with null editor\n");
+    return;
+  }
+
+  song=editor->song;
 
   if(editor->section>=song->numsections)
     editor->section=0;
@@ -258,3 +304,67 @@ void editor_refresh_playseq_and_block(struct editor *editor) {
   } else
     editor->block=song->playseqs[editor->playseq]->blocknumbers[editor->position];
 }
+
+void editor_midi_input(struct editor *editor, unsigned char *data) {
+  Tracker *tracker;
+  struct song *song;
+
+  if(editor==NULL) {
+    fprintf(stderr, "editor_midi_input() called with null editor\n");
+    return;
+  }
+
+  tracker=(Tracker *)glade_xml_get_widget(editor->gui->xml, "gui_tracker");
+  song=editor->song;
+
+  switch(data[0]&0xf0) {
+  case 0x80:
+    /* Note off */
+    if(editor->edit) {
+      if(editor->chord) {
+	editor->chordstatus--;
+	tracker_step_cursor_channel(tracker, -1);
+	if(editor->chordstatus==0)
+	  editor_set_line_immediate(editor, editor->line+editor->space);
+      }
+    }
+    break;
+  case 0x90:
+    /* Note on */
+    if(editor->edit) {
+      block_set_note(song->blocks[editor->block], tracker->patpos,
+		     tracker->cursor_ch, data[1]/12, data[1]%12+1,
+		     editor->instrument+1);
+      block_set_command_full(song->blocks[editor->block], tracker->patpos,
+			     tracker->cursor_ch, editor->cmdpage,
+			     COMMAND_VELOCITY, data[2]);
+      tracker_redraw_row(tracker, tracker->patpos);
+      if(editor->chord) {
+	editor->chordstatus++;
+	tracker_step_cursor_channel(tracker, 1);
+      } else
+	editor_set_line_immediate(editor, editor->line+editor->space);
+    }
+    break;
+  case 0xb0:
+    /* MIDI controller */
+    if(editor->edit) {
+      block_set_command_full(song->blocks[editor->block], tracker->patpos,
+			     tracker->cursor_ch, editor->cmdpage,
+			     COMMAND_MIDI_CONTROLLERS+data[1], data[2]);
+      tracker_redraw_row(tracker, tracker->patpos);
+    }
+    break;
+  case 0xe0:
+    /* Pitch wheel */
+    if(editor->edit) {
+      block_set_command_full(song->blocks[editor->block], tracker->patpos,
+			     tracker->cursor_ch, editor->cmdpage,
+			     COMMAND_PITCH_WHEEL, data[2]);
+      tracker_redraw_row(tracker, tracker->patpos);
+    }
+    break;
+  default:
+    break;
+  }
+}
