@@ -79,7 +79,7 @@ void instrument_save(struct instrument *instrument, int number,
 		     xmlNodePtr parent) {
   char c[10];
 
-  xmlNodePtr node, lb;
+  xmlNodePtr node;
 
   node=xmlNewChild(parent, NULL, "instrument", instrument->name);
   snprintf(c, 10, "%d", number);
@@ -95,6 +95,5 @@ void instrument_save(struct instrument *instrument, int number,
   snprintf(c, 10, "%d", instrument->hold);
   xmlSetProp(node, "hold", c);
 
-  lb=xmlNewText("\n");
-  xmlAddChild(parent, lb);
+  xmlAddChild(parent, xmlNewText("\n"));
 }
