@@ -59,8 +59,16 @@ int main(int argc, char *argv[]) {
     /* Main loop */
     gdk_threads_enter();
     while(!gui_quit) {
+      unsigned char *data=NULL;
+
       while(gtk_events_pending())
 	gtk_main_iteration_do(FALSE);
+
+      while((data=midi_read(editor->player->midi))!=NULL) {
+	editor_midi_input(editor, data);
+	free(data);
+      }
+
       gui_refresh(editor->gui);
       usleep(40);
     }
