@@ -20,37 +20,41 @@
  */
 
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
 #include "midi.h"
 
 /* Opens a MIDI device */
-FILE *midi_open(char *filename) {
-  FILE *midi;
+int midi_open(char *filename) {
+  int midi;
 
-  /* Open MIDI device */
-  midi=fopen(filename, "r+");
-  if(midi==NULL)
+  /* Open MIDI device:read/write, do not block (for input), auto flush */
+  midi=open(filename, O_RDWR|O_NONBLOCK|O_SYNC);
+  if(midi==-1)
     fprintf(stderr, "Could not open MIDI device %s.\n", filename);
 
   return midi;
 }
 
 /* Closes a MIDI device */
-void midi_close(FILE *midi) {
-  if(midi==NULL) {
-    fprintf(stderr, "midi_close() called with null file\n");
+void midi_close(int midi) {
+  if(midi==-1) {
+    fprintf(stderr, "midi_close() called with invalid fd\n");
     return;
   }
 
-  fclose(midi);
+  close(midi);
 }
 
 /* Stops a note playing on a MIDI channel using requested velocity */
-void midi_note_off(FILE *midi, unsigned char channel, unsigned char note,
+void midi_note_off(int midi, unsigned char channel, unsigned char note,
 		   unsigned char velocity) {
   unsigned char data[3];
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_note_off() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_note_off() called with invalid fd\n");
     return;
   }
 
@@ -58,17 +62,16 @@ void midi_note_off(FILE *midi, unsigned char channel, unsigned char note,
   data[1]=note&0x7f;
   data[2]=velocity&0x7f;
   
-  fwrite(data, 3, 1, midi);
-  fflush(midi);
+  write(midi, data, 3);
 }
 
 /* Plays a note on a MIDI channel using requested velocity */
-void midi_note_on(FILE *midi, unsigned char channel, unsigned char note,
+void midi_note_on(int midi, unsigned char channel, unsigned char note,
 		   unsigned char velocity) {
   unsigned char data[3];
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_note_on() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_note_on() called with invalid fd\n");
     return;
   }
 
@@ -76,17 +79,16 @@ void midi_note_on(FILE *midi, unsigned char channel, unsigned char note,
   data[1]=note&0x7f;
   data[2]=velocity&0x7f;
   
-  fwrite(data, 3, 1, midi);
-  fflush(midi);
+  write(midi, data, 3);
 }
 
 /* Sets the aftertouch pressure of a note playing on a MIDI channel */
-void midi_aftertouch(FILE *midi, unsigned char channel, unsigned char note,
+void midi_aftertouch(int midi, unsigned char channel, unsigned char note,
 		     unsigned char pressure) {
   unsigned char data[3];
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_aftertouch() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_aftertouch() called with invalid fd\n");
     return;
   }
 
@@ -94,17 +96,16 @@ void midi_aftertouch(FILE *midi, unsigned char channel, unsigned char note,
   data[1]=note&0x7f;
   data[2]=pressure&0x7f;
   
-  fwrite(data, 3, 1, midi);
-  fflush(midi);
+  write(midi, data, 3);
 }
 
 /* Sets the MIDI controller value of a MIDI channel */
-void midi_controller(FILE *midi, unsigned char channel, 
+void midi_controller(int midi, unsigned char channel, 
 		     unsigned char controller, unsigned char value) {
   unsigned char data[3];
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_controller() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_controller() called with invalid fd\n");
     return;
   }
 
@@ -112,51 +113,48 @@ void midi_controller(FILE *midi, unsigned char channel,
   data[1]=controller&0x7f;
   data[2]=value&0x7f;
   
-  fwrite(data, 3, 1, midi);
-  fflush(midi);
+  write(midi, data, 3);
 }
 
 /* Send a program change on a MIDI channel */
-void midi_program_change(FILE *midi, unsigned char channel,
+void midi_program_change(int midi, unsigned char channel,
 			 unsigned char program) {
   unsigned char data[2];
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_program_change() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_program_change() called with invalid fd\n");
     return;
   }
 
   data[0]=0xc0|channel;
   data[1]=program&0x7f;
   
-  fwrite(data, 2, 1, midi);
-  fflush(midi);
+  write(midi, data, 2);
 }
 
 /* Sets the channel pressure of a MIDI channel */
-void midi_channel_pressure(FILE *midi, unsigned char channel,
+void midi_channel_pressure(int midi, unsigned char channel,
 			   unsigned char pressure) {
   unsigned char data[2];
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_channel_pressure() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_channel_pressure() called with invalid fd\n");
     return;
   }
 
   data[0]=0xd0|channel;
   data[1]=pressure&0x7f;
   
-  fwrite(data, 2, 1, midi);
-  fflush(midi);
+  write(midi, data, 2);
 }
 
 /* Sets the pitch wheel value of a MIDI channel */
-void midi_pitch_wheel(FILE *midi, unsigned char channel,
+void midi_pitch_wheel(int midi, unsigned char channel,
 		      unsigned short value) {
   unsigned char data[3];
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_pitch_wheel() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_pitch_wheel() called with invalid fd\n");
     return;
   }
 
@@ -164,17 +162,16 @@ void midi_pitch_wheel(FILE *midi, unsigned char channel,
   data[1]=(value>>7)&0x7f;
   data[2]=value&0x7f;
   
-  fwrite(data, 3, 1, midi);
-  fflush(midi);
+  write(midi, data, 3);
 }
 
 /* Sends a MIDI SysEX message */
-void midi_system_exclusive(FILE *midi, unsigned char *message,
+void midi_system_exclusive(int midi, unsigned char *message,
 			   unsigned short length) {
   unsigned char *data=(unsigned char *)calloc(length+2, sizeof(unsigned char));
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_system_exclusive() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_system_exclusive() called with invalid fd\n");
     return;
   }
 
@@ -182,31 +179,69 @@ void midi_system_exclusive(FILE *midi, unsigned char *message,
   memcpy(data+1, message, length);
   data[length+1]=0xf7;
   
-  fwrite(data, length+2, 1, midi);
-  fflush(midi);
+  write(midi, data, length+2);
 
   free(data);
 }
 
 /* Send a clock message */
-void midi_clock(FILE *midi) {
-  if(midi==NULL) {
-    fprintf(stderr, "midi_clock() called with null file\n");
+void midi_clock(int midi) {
+  unsigned char data[]={ 0xf8 };
+
+  if(midi==-1) {
+    fprintf(stderr, "midi_clock() called with invalid fd\n");
     return;
   }
 
-  fputc(0xf8, midi);
-  fflush(midi);
+  write(midi, data, 1);
 }
 
+/* Receives a MIDI message */
+unsigned char *midi_read(int midi) {
+  int status=1;
+  unsigned char c=0, *d=NULL;
+
+  if(midi==-1) {
+    fprintf(stderr, "midi_read() called with invalid fd\n");
+    return NULL;
+  }
+
+  while((status=read(midi, &c, 1))==1 && c<0x80);
+
+  if(status!=1)
+    return NULL;
+  
+  switch(c&0xf0) {
+  case 0x80:
+  case 0x90:
+  case 0xa0:
+  case 0xb0:
+  case 0xe0: {
+    d=(unsigned char *)calloc(3, sizeof(unsigned char));
+    d[0]=c;
+    status=read(midi, d+1, 2);
+
+    if(status==2)
+      return d;
+    else {
+      free(d);
+      return NULL;
+    }
+  }
+  default:
+    break;
+  }
+  return NULL;
+}
 
 /* Receives a MIDI SysEx message */
-void midi_read_system_exclusive(FILE *midi, struct sysex *sysex,
+void midi_read_system_exclusive(int midi, struct sysex *sysex,
 				int autostop) {
   int status=0, i=0;
+  unsigned char c=0;
 
-  if(midi==NULL) {
-    fprintf(stderr, "midi_read_system_exclusive() called with null file\n");
+  if(midi==-1) {
+    fprintf(stderr, "midi_read_system_exclusive() called with invalid fd\n");
     return;
   }
 
@@ -216,23 +251,20 @@ void midi_read_system_exclusive(FILE *midi, struct sysex *sysex,
   }
     
   /* Read until a SysEx dump begins */
-  while(status!=0xf0 && status!=EOF)
-    status=fgetc(midi);
-  
-  /* Return on error */
-  if(status==EOF)
-    return;
+  while(c!=0xf0) {
+    usleep(40);
+    status=read(midi, &c, 1);
+  }
   
   /* Read until the buffer is full or to the end of the SysEx (autostop) */
   while(i<sysex->length) {
-    status=fgetc(midi);
+    status=read(midi, &c, 1);
     
-    if(status==EOF)
-      break;
-    
-    sysex->data[i++]=status;
-    if(autostop==1 && sysex->data[i-1]==0xf7)
-      break;
+    if(status==1) {
+      sysex->data[i++]=c;
+      if(autostop==1 && sysex->data[i-1]==0xf7)
+	break;
+    }
   }
   
   /* Remove unnecessary bytes if necessary */
@@ -299,7 +331,7 @@ struct sysex *sysex_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 /* Saves a SysEx message to an XML file */
 void sysex_save(struct sysex *sysex, int number, xmlNodePtr parent) {
   char *c, t[10];
-  xmlNodePtr node, lb;
+  xmlNodePtr node;
   int i;
 
   if(sysex==NULL) {
@@ -318,8 +350,7 @@ void sysex_save(struct sysex *sysex, int number, xmlNodePtr parent) {
   snprintf(t, 10, "%d", sysex->autosend);
   xmlSetProp(node, "autosend", t);
 
-  lb=xmlNewText("\n");
-  xmlAddChild(parent, lb);
+  xmlAddChild(parent, xmlNewText("\n"));
 
   free(c);
 }
