@@ -422,6 +422,21 @@ struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset) {
     /* FIX: IMPLEMENT THIS */
   }
 
+  if(getlong(data+64)!=0) {
+    mmd0exp->mmdcmd3x=(struct MMDMIDICmd3x *)calloc(1, sizeof(struct MMDMIDICmd3x));
+    data2=base+getlong(data+64);
+    mmd0exp->mmdcmd3x->struct_vers=getbyte(data2);
+    mmd0exp->mmdcmd3x->num_of_settings=getword(data2+2);
+    mmd0exp->mmdcmd3x->ctrlr_types=(unsigned char *)calloc(mmd0exp->mmdcmd3x->num_of_settings, sizeof(unsigned char));
+    mmd0exp->mmdcmd3x->ctrlr_numbers=(unsigned short *)calloc(mmd0exp->mmdcmd3x->num_of_settings, sizeof(unsigned short));
+    data3=base+getlong(data2+4);
+    for(i=0; i<mmd0exp->mmdcmd3x->num_of_settings; i++)
+      mmd0exp->mmdcmd3x->ctrlr_types[i]=getbyte(data3+i);
+    data3=base+getlong(data2+8);
+    for(i=0; i<mmd0exp->mmdcmd3x->num_of_settings; i++)
+      mmd0exp->mmdcmd3x->ctrlr_numbers[i]=getword(data3+i*2);
+  }
+
   return mmd0exp;
 }
 
