@@ -124,8 +124,15 @@ static void player_next_tick(struct player *player) {
 	midichannel=song->instruments[instrument-1]->midichannel;
       else
 	/* Note playing defines MIDI channel */
-	midichannel=player->trackstatus[i*4+TRACK_STATUS_MIDI_CHANNEL];
+	midichannel=player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL];
 
+      /* Check for previous command if any */
+      if(command==COMMAND_PREVIOUS_COMMAND_VALUE) {
+	if(value!=0)
+	  command=player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_PREVIOUS_COMMAND];
+      } else
+	player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_PREVIOUS_COMMAND]=command;
+      
       switch(command) {
       case COMMAND_PITCH_WHEEL:
 	/* Pitch wheel can be set if the MIDI channel is known */
@@ -189,28 +196,28 @@ static void player_next_tick(struct player *player) {
 	    player->midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]=value;
 	} else {
 	  /* Note playing defines MIDI channel */
-	  midichannel=player->trackstatus[i*4+TRACK_STATUS_MIDI_CHANNEL];
+	  midichannel=player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL];
 
 	  if(midichannel!=-1) {
 	    if(value<0x80) {
 	      if(player->tick==0) {
 		if(value>0) {
-		  midi_aftertouch(player->midi, midichannel, player->trackstatus[i*4+TRACK_STATUS_NOTE], value);
+		  midi_aftertouch(player->midi, midichannel, player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE], value);
 		  player->midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]=value;
 		} else {
 		  midi_note_off(player->midi, midichannel,
-				player->trackstatus[i*4+TRACK_STATUS_NOTE],
+				player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE],
 				127);
-		  player->trackstatus[i*4+TRACK_STATUS_NOTE]=-1;
+		  player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE]=-1;
 		}
 	      }
 	    } else {
 	      if(player->tick<song->ticksperline-1) {
 		int delta=(value-0x80-player->midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH])/((int)song->ticksperline-1);
 		
-		midi_aftertouch(player->midi, midichannel, player->trackstatus[i*4+TRACK_STATUS_NOTE], player->midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]+player->tick*delta);
+		midi_aftertouch(player->midi, midichannel, player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE], player->midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]+player->tick*delta);
 	      } else {
-		midi_aftertouch(player->midi, midichannel, player->trackstatus[i*4+TRACK_STATUS_NOTE], value-0x80);
+		midi_aftertouch(player->midi, midichannel, player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE], value-0x80);
 		player->midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]=value-0x80;
 	      }
 	    }
@@ -284,22 +291,22 @@ static void player_next_tick(struct player *player) {
 	hold=song->instruments[b->notes[(b->tracks*player->line+i)*2+1]-1]->hold;
 
       if(hold==0)
-	player->trackstatus[i*4+TRACK_STATUS_HOLD]=-1;
+	player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]=-1;
       else
-	player->trackstatus[i*4+TRACK_STATUS_HOLD]=hold;
+	player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]=hold;
     }
   }
 
   /* Decrement hold times of notes and stop notes that should be stopped */
   for(i=0; i<song->maxtracks; i++) {
-    if(player->trackstatus[i*4+TRACK_STATUS_HOLD]!=-1) {
-      player->trackstatus[i*4+TRACK_STATUS_HOLD]--;
-      if(player->trackstatus[i*4+TRACK_STATUS_HOLD]==-1) {
+    if(player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]!=-1) {
+      player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]--;
+      if(player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]==-1) {
 	midi_note_off(player->midi,
-		      player->trackstatus[i*4+TRACK_STATUS_MIDI_CHANNEL],
-		      player->trackstatus[i*4+TRACK_STATUS_NOTE],
+		      player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL],
+		      player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE],
 		      127);
-	player->trackstatus[i*4+TRACK_STATUS_NOTE]=-1;
+	player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE]=-1;
       }
     }
   }
@@ -377,10 +384,10 @@ void player_play_note(struct player *player,
   song=player->song;
 
   /* Stop currently playing note */
-  if(player->trackstatus[track*4+TRACK_STATUS_NOTE]!=-1) {
+  if(player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE]!=-1) {
     midi_note_off(player->midi,
-		  player->trackstatus[track*4+TRACK_STATUS_MIDI_CHANNEL],
-		  player->trackstatus[track*4+TRACK_STATUS_NOTE],
+		  player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL],
+		  player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE],
 		  127);
   }
 
@@ -390,23 +397,23 @@ void player_play_note(struct player *player,
     note+=song->instruments[instrument]->transpose;
 
     /* Set new track status (channel, note, volume, time) */
-    player->trackstatus[track*4+TRACK_STATUS_MIDI_CHANNEL]=song->instruments[instrument]->midichannel;
-    player->trackstatus[track*4+TRACK_STATUS_NOTE]=note;
-    player->trackstatus[track*4+TRACK_STATUS_VOLUME]=song->instruments[instrument]->defaultvelocity*volume/127*(song->trackvolumes[track]&127)/127*((255-song->trackvolumes[track])&128)/128*song->mastervolume/127;
+    player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL]=song->instruments[instrument]->midichannel;
+    player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE]=note;
+    player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_VOLUME]=song->instruments[instrument]->defaultvelocity*volume/127*(song->trackvolumes[track]&127)/127*((255-song->trackvolumes[track])&128)/128*song->mastervolume/127;
     if(song->instruments[instrument]->hold>0)
-      player->trackstatus[track*4+TRACK_STATUS_HOLD]=song->instruments[instrument]->hold;
+      player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]=song->instruments[instrument]->hold;
     else
-      player->trackstatus[track*4+TRACK_STATUS_HOLD]=-1;
+      player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]=-1;
 
     /* Make sure the volume isn't too large */
-    if(player->trackstatus[track*4+TRACK_STATUS_VOLUME]<0)
-      player->trackstatus[track*4+TRACK_STATUS_VOLUME]=127;
+    if(player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_VOLUME]<0)
+      player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_VOLUME]=127;
     
     /* Play note */
     midi_note_on(player->midi,
-		 player->trackstatus[track*4+TRACK_STATUS_MIDI_CHANNEL],
-		 player->trackstatus[track*4+TRACK_STATUS_NOTE],
-		 player->trackstatus[track*4+TRACK_STATUS_VOLUME]);
+		 player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL],
+		 player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE],
+		 player->trackstatus[track*TRACK_STATUS_NUMVALUES+TRACK_STATUS_VOLUME]);
   }
 }
 
@@ -420,19 +427,15 @@ void player_stop_notes(struct player *player) {
   }
 
   for(i=0; i<player->song->maxtracks; i++) {
-    if(player->trackstatus[i*4+1]!=-1) {
+    if(player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE]!=-1) {
       midi_note_off(player->midi,
-		    player->trackstatus[i*4], player->trackstatus[i*4+1], 127);
-      player->trackstatus[i*4]=-1;
-      player->trackstatus[i*4+1]=-1;
-      player->trackstatus[i*4+2]=-1;
-      player->trackstatus[i*4+3]=-1;
+		    player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL], player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE], 127);
+      player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL]=-1;
+      player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE]=-1;
+      player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_VOLUME]=-1;
+      player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]=-1;
     }
   }
-
-  /* Reset MIDI controller values to 0x40 (can't be horribly wrong...) */
-  for(i=0; i<16*VALUES; i++)
-    player->midicontrollervalues[i]=0x40;
 }
 
 /* Stops all notes playing at the moment */
@@ -538,6 +541,10 @@ struct player *player_open(struct song *song, struct editor *editor) {
 			    player->song->sysexes[i]->length);
   }
 
+  /* Reset MIDI controller values */
+  for(i=0; i<16*VALUES; i++)
+    player->midicontrollervalues[i]=0;
+
   return player;
 }
 
@@ -635,13 +642,14 @@ void player_trackstatus_create(struct player *player, int oldmax) {
   }
 
   player->trackstatus=(char *)realloc(player->trackstatus,
-				      4*player->song->maxtracks*sizeof(char));
+				      TRACK_STATUS_NUMVALUES*player->song->maxtracks*sizeof(char));
   
   /* Set new tracks to -1 */
   for(i=oldmax; i<player->song->maxtracks; i++) {
-    player->trackstatus[i*4]=-1;
-    player->trackstatus[i*4+1]=-1;
-    player->trackstatus[i*4+2]=-1;
-    player->trackstatus[i*4+3]=-1;
+    player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_MIDI_CHANNEL]=-1;
+    player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_NOTE]=-1;
+    player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_VOLUME]=-1;
+    player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_HOLD]=-1;
+    player->trackstatus[i*TRACK_STATUS_NUMVALUES+TRACK_STATUS_PREVIOUS_COMMAND]=0;
   }
 }
