@@ -466,21 +466,21 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
   /* Create a table for converting MMD commands; values need special handling */
   for(i=0; i<256; i++)
     commandconversion[i]=i;
-  commandconversion[0]=COMMAND_MIDI_COMMAND_VALUE;
+  commandconversion[0]=COMMAND_PREVIOUS_COMMAND_VALUE;
   commandconversion[1]=COMMAND_PITCH_WHEEL;
   commandconversion[2]=COMMAND_PITCH_WHEEL;
   commandconversion[3]=COMMAND_PITCH_WHEEL;
-  commandconversion[4]=COMMAND_NOT_DEFINED;
+  commandconversion[4]=COMMAND_MIDI_CONTROLLERS+1;
   commandconversion[5]=COMMAND_NOT_DEFINED;
   commandconversion[6]=COMMAND_NOT_DEFINED;
   commandconversion[7]=COMMAND_NOT_DEFINED;
-  commandconversion[8]=COMMAND_NOT_DEFINED;
+  commandconversion[8]=COMMAND_HOLD;
   commandconversion[9]=COMMAND_TPL;
   commandconversion[10]=COMMAND_VELOCITY;
   commandconversion[11]=COMMAND_PLAYSEQ_POSITION;
   commandconversion[12]=COMMAND_VELOCITY;
   commandconversion[13]=COMMAND_CHANNEL_PRESSURE;
-  commandconversion[14]=COMMAND_NOT_DEFINED;
+  commandconversion[14]=COMMAND_MIDI_CONTROLLERS+10;
   commandconversion[15]=COMMAND_TEMPO;
   commandconversion[16]=COMMAND_SEND_SYSEX;
   commandconversion[17]=COMMAND_NOT_DEFINED;
@@ -488,14 +488,14 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
   commandconversion[19]=COMMAND_NOT_DEFINED;
   commandconversion[20]=COMMAND_NOT_DEFINED;
   commandconversion[21]=COMMAND_NOT_DEFINED;
-  commandconversion[22]=COMMAND_NOT_DEFINED;
+  commandconversion[22]=COMMAND_MIDI_CONTROLLERS+7;
   commandconversion[23]=COMMAND_NOT_DEFINED;
   commandconversion[24]=COMMAND_NOT_DEFINED;
   commandconversion[25]=COMMAND_NOT_DEFINED;
   commandconversion[26]=COMMAND_NOT_DEFINED;
   commandconversion[27]=COMMAND_NOT_DEFINED;
   commandconversion[28]=COMMAND_NOT_DEFINED;
-  commandconversion[29]=COMMAND_NOT_DEFINED;
+  commandconversion[29]=COMMAND_END_BLOCK;
   commandconversion[30]=COMMAND_NOT_DEFINED;
   commandconversion[31]=COMMAND_DELAY;
 
@@ -554,7 +554,8 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
 	  block->commands[(j*block->tracks+k)*2]=commandconversion[notes[(j*block->tracks+k)*4+2]];
 	  block->commands[(j*block->tracks+k)*2+1]=notes[(j*block->tracks+k)*4+3];
 	  /* Command values may need special handling */
-	  song_convert_commands(&(block->commands[(j*block->tracks+k)*2]));
+	  song_convert_commands(&(block->commands[(j*block->tracks+k)*2]),
+				mmd->expdata->mmdcmd3x);
 	}
 
       if(MMD1Block->info->pagetable) {
@@ -565,7 +566,7 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
 	      block->commands[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2+1]=MMD1Block->info->pagetable->page[j][(k*MMD1Block->numtracks+l)*2+1];
 
 	      /* Command values may need special handling */
-	      song_convert_commands(&(block->commands[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]));
+	      song_convert_commands(&(block->commands[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]), mmd->expdata->mmdcmd3x);
 	    }
 	}
       }
@@ -649,7 +650,8 @@ void song_transpose(struct song *song, int instruments, int halfnotes) {
 }
 
 /* Converts MMD command values at given location */
-void song_convert_commands(unsigned char *command) {
+void song_convert_commands(unsigned char *command,
+			   struct MMDMIDICmd3x *mmdcmd3x) {
   switch(command[0]) {
   case COMMAND_VELOCITY:
     /* Fix volume range to 0..127 */
@@ -666,40 +668,40 @@ void song_convert_commands(unsigned char *command) {
       command[0]=COMMAND_RETRIGGER;
     break;
   case COMMAND_TEMPO:
-    /* Values >240 have special meanings in OctaMED */
-    if(command[1]>240) {
+    /* Values >0xf0 have special meanings in OctaMED */
+    if(command[1]>0xf0) {
       switch(command[1]) {
-      case 241:
+      case 0xf1:
 	/* Makes a single note play twice */
 	command[0]=COMMAND_RETRIGGER;
 	command[1]=3;
 	break;
-      case 242:
+      case 0xf2:
 	/* Delays the start of a note by half a line */
 	command[0]=COMMAND_DELAY;
 	command[1]=3;
 	break;
-      case 243:
+      case 0xf3:
 	/* Works like 0FF1 except the note is played three times */
 	command[0]=COMMAND_RETRIGGER;
 	command[1]=2;
 	break;
-      case 244:
+      case 0xf4:
 	/* Delays the note one-third of a line */
 	command[0]=COMMAND_DELAY;
 	command[1]=2;
 	break;
-      case 245:
+      case 0xf5:
 	/* Delays the note two thirds of a line */
 	command[0]=COMMAND_DELAY;
 	command[1]=4;
 	break;
-      case 254:
+      case 0xfe:
 	/* Stops the song playing */
 	command[0]=COMMAND_STOP;
 	command[1]=0;
 	break;
-      case 255:
+      case 0xff:
 	/* Stops the note on the current track */
 	command[0]=COMMAND_VELOCITY;
 	command[1]=0;
@@ -711,6 +713,26 @@ void song_convert_commands(unsigned char *command) {
       }
     }
     break;
+  case 0x31:
+  case 0x32:
+  case 0x33:
+  case 0x34:
+  case 0x35:
+  case 0x36:
+  case 0x37:
+  case 0x38:
+  case 0x39:
+  case 0x3a:
+  case 0x3b:
+  case 0x3c:
+  case 0x3d:
+  case 0x3e:
+  case 0x3f:
+    if(mmdcmd3x->ctrlr_types[command[0]-0x31]==MCS_TYPE_STD_MSB)
+      command[0]=mmdcmd3x->ctrlr_numbers[command[0]-0x31];
+    break;
+  default:
+    break;
   }
 }
 
@@ -988,7 +1010,7 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 /* Saves a song to an XML file */
 void song_save(struct song *song, char *filename) {
   xmlDocPtr doc;
-  xmlNodePtr node, subnode, lb;
+  xmlNodePtr node, subnode;
   int i;
   char c[10];
 
@@ -1008,74 +1030,59 @@ void song_save(struct song *song, char *filename) {
   xmlSetProp(doc->xmlRootNode, "mastervolume", c);
   snprintf(c, 10, "%d", song->sendsync);
   xmlSetProp(doc->xmlRootNode, "sendsync", c);
-  lb=xmlNewText("\n\n");
-  xmlAddChild(doc->xmlRootNode, lb);
+  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
   node=xmlNewChild(doc->xmlRootNode, NULL, "blocks", NULL);
-  lb=xmlNewText("\n");
-  xmlAddChild(node, lb);
+  xmlAddChild(node, xmlNewText("\n"));
   /* Add all blocks */
   for(i=0; i<song->numblocks; i++)
     block_save(song->blocks[i], i, node);
-  lb=xmlNewText("\n\n");
-  xmlAddChild(doc->xmlRootNode, lb);
+  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
   node=xmlNewChild(doc->xmlRootNode, NULL, "sections", NULL);
-  lb=xmlNewText("\n");
-  xmlAddChild(node, lb);
+  xmlAddChild(node, xmlNewText("\n"));
   /* Add all sections */
   for(i=0; i<song->numsections; i++) {
     snprintf(c, 10, "%d", song->sections[i]);
     subnode=xmlNewChild(node, NULL, "section", c);
     snprintf(c, 10, "%d", i);
     xmlSetProp(subnode, "number", c);
-    lb=xmlNewText("\n");
-    xmlAddChild(node, lb);
+    xmlAddChild(node, xmlNewText("\n"));
   }
-  lb=xmlNewText("\n\n");
-  xmlAddChild(doc->xmlRootNode, lb);
+  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
   node=xmlNewChild(doc->xmlRootNode, NULL, "playingsequences", NULL);
-  lb=xmlNewText("\n");
-  xmlAddChild(node, lb);
+  xmlAddChild(node, xmlNewText("\n"));
   /* Add all playing sequences */
   for(i=0; i<song->numplayseqs; i++)
     playseq_save(song->playseqs[i], i, node);
-  lb=xmlNewText("\n\n");
-  xmlAddChild(doc->xmlRootNode, lb);
+  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
   node=xmlNewChild(doc->xmlRootNode, NULL, "instruments", NULL);
-  lb=xmlNewText("\n");
-  xmlAddChild(node, lb);
+  xmlAddChild(node, xmlNewText("\n"));
   /* Add all instruments */
   for(i=0; i<song->numinstruments; i++)
     instrument_save(song->instruments[i], i, node);
-  lb=xmlNewText("\n\n");
-  xmlAddChild(doc->xmlRootNode, lb);
+  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
   node=xmlNewChild(doc->xmlRootNode, NULL, "trackvolumes", NULL);
-  lb=xmlNewText("\n");
-  xmlAddChild(node, lb);
+  xmlAddChild(node, xmlNewText("\n"));
   /* Add all track volumes */
   for(i=0; i<song->maxtracks; i++) {
     snprintf(c, 10, "%d", song->trackvolumes[i]);
     subnode=xmlNewChild(node, NULL, "trackvolume", c);
     snprintf(c, 10, "%d", i);
     xmlSetProp(subnode, "track", c);
-    lb=xmlNewText("\n");
-    xmlAddChild(node, lb);
+    xmlAddChild(node, xmlNewText("\n"));
   }
-  lb=xmlNewText("\n\n");
-  xmlAddChild(doc->xmlRootNode, lb);
+  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
   node=xmlNewChild(doc->xmlRootNode, NULL, "sysexes", NULL);
-  lb=xmlNewText("\n");
-  xmlAddChild(node, lb);
+  xmlAddChild(node, xmlNewText("\n"));
   /* Add all SysEx messages */
   for(i=0; i<song->numsysexes; i++)
     sysex_save(song->sysexes[i], i, node);
-  lb=xmlNewText("\n\n");
-  xmlAddChild(doc->xmlRootNode, lb);
+  xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
   /* Use stdout if no filename is specified */
   if(filename==NULL)
